import requests
import subprocess
import sys
import json
from SPARQLWrapper import SPARQLWrapper, JSON


def make_repo(config, url):
    headers = {}
    files = {"config": (config, open(config, "rb"))}
    response = requests.post(f"{url}/rest/repositories", headers=headers, files=files)
    return response


def ls_repo(url):
    headers = {"Accept": "application/json"}
    response = requests.get(f"{url}/rest/repositories", headers=headers)
    return response


def rm_repo(url, repo_name):
    headers = {"Accept": "application/json"}
    response = requests.delete(f"{url}/rest/repositories/{repo_name}", headers=headers)
    return response


def turtle_to_deletion_sparql(turtle):
    """
    Translates a turtle file into a SPARQL statement deleting the triples in the file

    extract prefix statements
    replace '@prefix' with 'prefix', case insenstive
    """

    prefixes = []
    body = []

    for line in turtle:
        line = line.strip()
        if len(line) > 0 and line[0] == "@":
            # translates '@prefix f: <whatever> .' to 'prefix f: <whatever>'
            prefixes.append(line[1:-1])
        else:
            body.append(line)

    prefix_str = "\n".join(prefixes)
    body_str = "\n".join(body)

    sparql = f"{prefix_str}\nDELETE DATA {{\n{body_str}\n}}"

    return sparql

def rm_data(url, repo_name, turtle_files):
    graphdb_url = f"{url}/repositories/{repo_name}/statements"
    for turtle in turtle_files:
        with open(turtle, "r") as f:
            turtle_lines = f.readlines()
            sparql_delete = turtle_to_deletion_sparql(turtle_lines)
            sparql = SPARQLWrapper(graphdb_url)
            sparql.method = "POST"
            sparql.queryType = "DELETE"
            sparql.setQuery(sparql_delete)
            sparql.query()

def update(url, repo_name, sparql_file):
    graphdb_url = f"{url}/repositories/{repo_name}/statements"
    sparql = SPARQLWrapper(graphdb_url)
    with open(sparql_file, "r") as fh:
        sparql_str = fh.read()
        sparql.setQuery(sparql_str)
        sparql.setReturnFormat(JSON)
        sparql.method = "POST"
        results = sparql.query().convert()
    return results


def sparql_query(url, repo_name, sparql_file):
    graphdb_url = f"{url}/repositories/{repo_name}"
    sparql = SPARQLWrapper(graphdb_url)

    with open(sparql_file, "r") as fh:
        sparql_str = fh.read()
        sparql.setQuery(sparql_str)
        sparql.setReturnFormat(JSON)
        results = sparql.query().convert()
    return results





def load_data(url, repo_name, turtle_files):
    headers = {"Content-Type": "application/json", "Accept": "application/json"}

    data = dict(
        fileNames=turtle_files,
        importSettings=dict(
            parserSettings=dict(
                # If True, filenames such as "cdc_cvv.ttl" will fail since '_' is an
                # invalid character in a URI
                verifyURISyntax=False
            )
        ),
    )

    rest_url = f"{url}/rest/data/import/server/{repo_name}"
    response = requests.post(rest_url, headers=headers, data=json.dumps(data))
    return response


def list_files(url, repo_name):
    rest_url = f"{url}/rest/data/import/server/{repo_name}"
    response = requests.get(rest_url)
    return response


def start_graphdb(path=None):
    cmd = "graphdb"
    if path:
        cmd = os.path.join(path, "graphdb")
    try:
        subprocess.run(f"{cmd} -sd")
    except FileNotFoundError:
        msg = f"Could not find executable `{cmd}`, please place it in PATH"
        print(msg, sys.stderr())
        sys.exit(1)



#!/usr/bin/env python3

import argparse
import textwrap
import json
import sys
import pgraphdb as cmd
import pgraphdb.cli as cli


parser = argparse.ArgumentParser(
    prog="pgraphdb",
    formatter_class=cli.SubcommandHelpFormatter,
    description="Wrapper around the GraphDB REST interface",
    epilog=textwrap.dedent("ladida back end stuff"),
)
subparsers = parser.add_subparsers(metavar="<subcommand>", title="subcommands")
subcommand = cli.subcommand_maker(subparsers)

def handle_response(response):
    if response.status_code >= 400:
        print(f"ERROR: {response.status_code}: {response.text}", file=sys.stderr)
        return None
    else:
        return response.text


@subcommand(
    [
        "start",
        cli.argument("config_file"),
        cli.argument("--path", help="The path to the GraphDB bin directory"),
    ]
)
def call_start_graphdb(args):
    """
    Start a GraphDB daemon in server mode
    """
    start_graphdb(path=args.path)


@subcommand(
    [
        "make",
        cli.argument("config_file"),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_make_repo(args):
    """
    Create a new data repository within a graphdb database
    """
    print(handle_response(cmd.make_repo(config=args.config_file, url=args.url)))


@subcommand(
    ["ls_repo", cli.argument("--url", help="GraphDB URL", default="http://localhost:7200")]
)
def call_ls_repo(args):
    """
    List all repositories in the GraphDB database
    """
    print(handle_response(cmd.ls_repo(url=args.url)))


@subcommand(
    [
        "rm_repo",
        cli.argument("repo_name", help="Repository name"),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_rm_repo(args):
    """
    Delete a repository in the GraphDB database
    """
    print(handle_response(cmd.rm_repo(repo_name=args.repo_name, url=args.url)))


@subcommand(
    [
        "rm_data",
        cli.argument("repo_name", help="Repository name"),
        cli.argument("turtle_files", help="Turtle files", nargs="*"),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_rm_data(args):
    """
    Delete all triples listed in the given turtle files 
    """
    cmd.rm_data(url=args.url, repo_name=args.repo_name, turtle_files=args.turtle_files)


@subcommand(
    [
        "update",
        cli.argument("repo_name", help="Repository name"),
        cli.argument("sparql_file", help="SPARQL file with DELETE or INSERT statement"),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_update(args):
    """
    Update database through delete or insert SPARQL query 
    """
    cmd.update(
        url=args.url, repo_name=args.repo_name, sparql_file=args.sparql_file
    )


@subcommand(
    [
        "ls_files",
        cli.argument("repo_name", help="Repository name"),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_ls_files(args):
    """
    List data files stored on the GraphDB server
    """
    json_str = handle_response(cmd.list_files(url=args.url, repo_name=args.repo_name))
    for entry in json.loads(json_str):
        print(entry["name"])


@subcommand(
    [
        "load",
        cli.argument("repo_name", help="Repository name"),
        cli.argument("turtle_files", help="Turtle files", nargs="*"),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_load_data(args):
    """
    load a given turtle file
    """
    print(
        handle_response(
            cmd.load_data(
                url=args.url, repo_name=args.repo_name, turtle_files=args.turtle_files
            )
        )
    )


@subcommand(
    [
        "query",
        cli.argument("repo_name", help="Repository name"),
        cli.argument("sparql_file", help="The SPARQL query file"),
        cli.argument(
            "--header",
            action="store_true",
            default=False,
            help="Print the header of column names",
        ),
        cli.argument("--url", help="GraphDB URL", default="http://localhost:7200"),
    ]
)
def call_sparql_query(args):
    """
    Submit a SPARQL query
    """

    def val(xs, field):
        if field in xs:
            return xs[field]["value"]
        else:
            return ""

    results = cmd.sparql_query(
        url=args.url, repo_name=args.repo_name, sparql_file=args.sparql_file
    )
    if args.header:
        print("\t".join(results["head"]["vars"]))
    for row in results["results"]["bindings"]:
        fields = (val(row, field) for field in results["head"]["vars"])
        print("\t".join(fields))


def main():
    args = parser.parse_args()
    if len(vars(args)) == 0:
        parser.print_help()
    else:
        args.func(args)


if __name__ == "__main__":
    main()



from setuptools import setup

from pgraphdb.version import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pgraphdb",
    version=__version__,
    description="A wrapper around the GraphDB REST interface",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/arendsee/pgraphdb",
    author="Zebulun Arendsee",
    author_email="zbwrnz@gmail.com",
    packages=["pgraphdb"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[ "SPARQLWrapper" ],
    entry_points={"console_scripts": ["pgraphdb=pgraphdb.ui:main"]},
    py_modules=["pgraphdb"],
    zip_safe=False,
)



import argparse
import textwrap

class SubcommandHelpFormatter(argparse.RawDescriptionHelpFormatter):
    """
    Remove the redundant "<subcommand>" string from under the "subcommands:"
    line in the help statement.

    Adapted from Jeppe Ledet-Pedersen on StackOverflow.
    """

    def _format_action(self, action):
        parts = super(argparse.RawDescriptionHelpFormatter, self)._format_action(action)
        if action.nargs == argparse.PARSER:
            parts = "\n".join(parts.split("\n")[1:])
        return parts

# subcommand decorator idea adapted from Mike Depalatis blog
def subcommand_maker(subparsers):
  def subcommand(args=[], parent=subparsers):
      def decorator(func):
          if func.__doc__:
              help_str = func.__doc__.strip().split("\n")[0]
              desc_str = textwrap.dedent(func.__doc__)
          else:
              help_str = "DOCUMENT ME PLEASE!!!"
              desc_str = None
          cmd_name = args[0]
          parser = parent.add_parser(
              cmd_name,
              description=desc_str,
              help=help_str,
              formatter_class=argparse.RawDescriptionHelpFormatter,
              #  usage=f"pgraphdb {cmd_name} <options>"
          )
          for arg in args[1:]:
              parser.add_argument(*arg[0], **arg[1])
          parser.set_defaults(func=func)

      return decorator
  return subcommand

def argument(*name_or_flags, **kwargs):
    return (list(name_or_flags), kwargs)
